#include<iostream>
#include<vector>
#define f first
#define s second
#define mp make_pair
using namespace std;



struct SegmentTree{
	vector<long long> data;
	vector<long long> toAdd;
	vector<pair<int, int> > segment;
	vector<bool> lazy;
	
	
	void init(int nSize) {
		int tpow = 1;
		while(tpow < nSize)
			tpow *= 2;
		
		data.resize(tpow*2, 0);
		toAdd.resize(tpow*2, 0);
		segment.resize(tpow*2);
		lazy.resize(tpow*2);
		
		//Initialize the segments
		for(int i=0;i<tpow;i++)
			segment[tpow+i] = mp(i, i+1);
		
		for(int i=tpow-1;i>0;i--)
			segment[i] = mp(segment[2*i].f, segment[2*i+1].s);
	}
	
	//Lazy propagation
	void propagate(size_t pos) {
		if(lazy[pos]) data[pos] = 0;
		data[pos] += toAdd[pos] * (segment[pos].s - segment[pos].f);
		
		if(2*pos < data.size() ) {
			if(lazy[pos]) {
				lazy[pos*2] = true;
				lazy[pos*2+1] = true;
				toAdd[pos*2] = 0;
				toAdd[pos*2+1] = 0;
			}
			
			toAdd[pos*2] += toAdd[pos];
			toAdd[pos*2+1] += toAdd[pos];
		}
		
		lazy[pos] = false;
		toAdd[pos] = 0;
	}
	
	
	//Operation here 1-add 2-set 3-get
	template<int operation>
	long long operate(int l, int r, int opval, int treePos = 1) {
		pair<int, int> &curSeg = segment[treePos];
		
		propagate(treePos);
		
		if(r <= curSeg.f || curSeg.s <= l)
			return 0;
		
		if(l <= curSeg.f && curSeg.s <= r) {
			if(operation == 1)
				toAdd[treePos] += opval;
			if(operation == 2) {
				toAdd[treePos] = opval;
				lazy[treePos] = true;
			}
			
			propagate(treePos);
			return data[treePos];
		}
		
		//Recursion
		long long ret = 	operate<operation>(l, r, opval, treePos*2) +
							operate<operation>(l, r, opval, treePos*2+1);
		
		data[treePos] = data[treePos*2] + data[treePos*2+1];
		
		return ret;
	}
	
	
	//Operation interfaces
	void add(int l, int r, int toadd) { operate<1>(l, r, toadd); }
	void set(int l, int r, int toadd) { operate<2>(l, r, toadd); }
	long long get(int l, int r) { return operate<3>(l, r, 0); } 
};








int main() {
	int n, m, p1, p2, p3, o;
	cin.sync_with_stdio(false);
	cin >> n >> m;
	
	SegmentTree stree;
	stree.init(n);
	
	
	for(int i=0;i<n;i++) {
		cin >> p1;
		stree.set(i, i+1, p1);
	}
	
	
	for(int i=0;i<m;i++) {
		cin >> o >> p1 >> p2;
		
		if(o == 1) {
			cin >> p3;
			stree.add(p1-1, p2, p3);
		}
		
		if(o == 2) {
			cin >> p3;
			stree.set(p1-1, p2, p3);
		}
		
		if(o == 3)
			cout << stree.get(p1-1, p2) << '\n';
	}
	
	return 0;
}
































