//Author: Oliver-Matis Lill
//Groups: 1,2,3,4,5
//Complexity: O(M log N)
//Concepts: DP, Segment Tree, Persistent Structures
//Description: This solution utilizes a persistent segment tree to
//caluclate and update DP values for each tree node.

#include<iostream>
#include<vector>
#define MOD 1000000007
#define MAX_N 100000
#define MAX_M 100000
using namespace std;


//Modular integer structure

struct Modint {
	int val;
	
	Modint(int nval = 0) {
		val = nval;
	}
	
	Modint& operator+=(Modint r) {
		val = (val + r.val) % MOD;
		return *this;
	}
	Modint operator+(Modint r) {return Modint(*this) += r;}
};


//Persistent segment tree structure

struct Node {
	Modint val;
	
	Node* lch, *rch;
	Node() {
		lch = 0;
		rch = 0;
	}
};


struct Stree {
	vector<Node> node;
	int n;
	
	Stree(int nsize) {
		n = 1;
		while(n < nsize)
			n *= 2;
		
		node.reserve((MAX_N + MAX_M) * 20);
		node.resize(2*n-1);
		
		for(int i=n-2;i>=0;i--) {
			node[i].lch = &node[2*i+1];
			node[i].rch = &node[2*i+2];
		}
	}
	
	
	Node* add(int dest, Modint toAdd, Node* cur, int lb = 0, int rb = -1) {
		if(rb == -1)
			rb = n;
		
		node.push_back(*cur);
		cur = &node.back();
		
		if(rb-lb == 1) {
			cur->val += toAdd;
			return cur;
		}
		
		int mid = (lb + rb) / 2;
		if(dest < mid)
			cur->lch = add(dest, toAdd, cur->lch, lb, mid);
		else
			cur->rch = add(dest, toAdd, cur->rch, mid, rb);
		
		cur->val = cur->lch->val + cur->rch->val;
		return cur;
	}
	
	
	Modint get(int l, int r, Node* cur, int lb = 0, int rb = -1) {
		if(rb == -1)
			rb = n;
		
		if(l <= lb && rb <= r)
			return cur->val;
		if(r <= lb || rb <= l)
			return 0;
		
		int mid = (lb + rb) / 2;
		return	get(l, r, cur->lch, lb, mid) +
				get(l, r, cur->rch, mid, rb);
	}
};



//Main function

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	
	int n;
	cin >> n;
	
	vector<int> p(n), T(n);
	
	for(int i=0;i<n;i++) {
		cin >> p[i];
		p[i]--;
	}
	for(int i=0;i<n;i++)
		cin >> T[i];
	
	
	//Initialize the persistent tree
	Stree stree(n);
	
	vector<Node*> subroot(n);
	subroot[0] = stree.add(0, 1, &stree.node[0]);
	
	for(int i=1;i<n;i++) {
		Modint combs = stree.get(0, T[i], subroot[p[i]]);
		subroot[i] = stree.add(T[i], combs, subroot[p[i]]);
	}
	
	//And finally answer the queries
	int m;
	cin >> m;
	
	for(int query = 0; query < m; query++) {
		int c, l, r;
		cin >> c >> l >> r;
		c--;
		
		cout << stree.get(l, r+1, subroot[c]).val << endl;
	}
	
	return 0;
}














































